SystemOrganization addCategory: #'Math-Core-Process'!

Object subclass: #DhbIterativeProcess
	instanceVariableNames: 'precision desiredPrecision maximumIterations result iterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Core-Process'!

!DhbIterativeProcess commentStamp: '<historical>' prior: 0!
A DhbIterativeProcess class is an abstract base class for processes which follow an iterative pattern. 

Subclasses of IterativeProcess will redefine
	initializeIterations
	evaluateIteration
	finalizeIterations

The instance variable result is used to store the most recent/best result, and is accessible through the result selector.

The maximumIterations: method allows control of the amount of work this method is allowed to do. Each evaluation of the iteration increments the instance variable iterations. When this number exceeds maximumIterations, the evaluate method will stop the process, and answer result.!

!DhbIterativeProcess class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultMaximumIterations
		"Private - Answers the default maximum number of iterations for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^50! !

!DhbIterativeProcess class methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
defaultPrecision
		"Private - Answers the default precision for newly created instances.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^DhbFloatingPointMachine new defaultNumericalPrecision! !

!DhbIterativeProcess class methodsFor: 'creation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
new
		"Create an instance of the class.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^super new initialize! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
desiredPrecision: aNumber
		"Defines the desired precision for the result.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	aNumber > 0
		ifFalse: [ ^self error: 'Illegal precision: ', aNumber printString].
	desiredPrecision := aNumber.! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluate
	"Perform the iteration until either the desired precision is attained or the number of iterations exceeds the maximum.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "

	iterations := 0.
	self initializeIterations.
	
	[iterations := iterations + 1.
	precision := self evaluateIteration.
	self hasConverged or: [iterations >= maximumIterations]] 
			whileFalse: [].
	self finalizeIterations.
	^self result! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
evaluateIteration
		"Dummy method (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^self subclassResponsibility! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
finalizeIterations
	"Perform cleanup operation if needed (must be implemented by subclass).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 9/3/99 "! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
hasConverged
		"
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 20/4/99 "
	^precision <= desiredPrecision! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initialize
		"Private - initialize the parameters of the receiver with default values.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	desiredPrecision := self class defaultPrecision.
	maximumIterations := self class defaultMaximumIterations.
	^self! !

!DhbIterativeProcess methodsFor: 'operation' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
initializeIterations
	"Initialize the iterations (must be implemented by subclass when needed).
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
iterations
		"Answers the number of iterations performed.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^iterations! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
limitedSmallValue: aNumber
		"Private - prevent aNumber from being smaller in absolute value than a small number.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 15/3/99 "
	^aNumber abs < DhbFloatingPointMachine new smallNumber
			ifTrue: [ DhbFloatingPointMachine new smallNumber]
			ifFalse:[ aNumber]! !

!DhbIterativeProcess methodsFor: 'initialization' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
maximumIterations: anInteger
		"Defines the maximum number of iterations.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	( anInteger isInteger and: [ anInteger > 1])
		ifFalse: [ ^self error: 'Invalid maximum number of iteration: ', anInteger printString].
	maximumIterations := anInteger.! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
precision
		"Answer the attained precision for the result.
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 7/1/99 "
	^precision! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
precisionOf: aNumber1 relativeTo: aNumber2
		"(c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 3/5/99 "
	^aNumber2 > DhbFloatingPointMachine new defaultNumericalPrecision
		ifTrue: [ aNumber1 / aNumber2]
		ifFalse:[ aNumber1]! !

!DhbIterativeProcess methodsFor: 'information' stamp: 'DanielUber 5/31/2012 12:05:19.968'!
result
		"Answer the result of the iterations (if any)
		 (c) Copyrights Didier BESSET, 1999, all rights reserved.
		 Initial code: 6/1/99 "
	^result! !
